/* 
 * glass - the openGL Articulated Skeleton System
 *
 * Copyright (C) 2001-2003  Robert Ancell <bob27@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <malloc.h>

#include "global.h"
#include "glass_transform.h"

/* Returns a new transform */
GlassTransform *_glassTransformNew(GlassTransformType type)
{
  GlassTransform *transform;

  transform = (GlassTransform *)malloc(sizeof(GlassTransform));
  if(transform == NULL)
  {
    fprintf(stderr, "Glass Error: Unable to allocate space for transform\n");
    return NULL;
  }
  memset(transform, 0, sizeof(GlassTransform));

  _glassTransformSetType(transform, type);

  return transform;
}

/* Get the number of parameters used by a transform */
int _glassTransformGetnParams(GlassTransformType type)
{
  switch(type)
  {
  case GLASS_ROTATE:
    return 4;
    break;
  case GLASS_TRANSLATE:
  case GLASS_SCALE:
    return 3;
    break;
  case GLASS_TRANSLATE_X:
  case GLASS_TRANSLATE_Y:
  case GLASS_TRANSLATE_Z:
  case GLASS_ROTATE_X:
  case GLASS_ROTATE_Y:
  case GLASS_ROTATE_Z:
  case GLASS_SCALE_X:
  case GLASS_SCALE_Y:
  case GLASS_SCALE_Z:
  case GLASS_SCALE_EVEN:
    return 1;
    break;
  default:
    return 0;
    break;
  }
}

/* Set the type of the transform, resizing arrays as necessary */
void _glassTransformSetType(GlassTransform *transform, GlassTransformType type)
{
  int i, old_nparams, nparams;

  old_nparams = _glassTransformGetnParams(transform->type);
  nparams = _glassTransformGetnParams(type);

  transform->type = type;

  /* If same number of parameters, nothing to change */
  if(old_nparams == nparams)
    return;

  /* Remove any constants that wont be pointed to after the realloc */
  for(i = nparams; i < old_nparams; i++)
    free(transform->values[i]);

  /* Extend/reduce the size of the arrays */
  if(nparams)
  {
    transform->variables = (GlassVariable **)realloc(transform->variables,
						     sizeof(GlassVariable *) * nparams);
    transform->values = (GLfloat **)realloc(transform->values,
					    sizeof(GLfloat *) * nparams);
  }
  else
  {
    free(transform->variables);
    free(transform->values);
    transform->variables = NULL;
    transform->values = NULL;
  }

  /* If only freed the memory because there are no parameters */
  if(nparams == 0)
  {
    transform->variables = NULL;
    transform->values = NULL;
  }

  /* Set the new parameters to constants of value zero */
  for(i = old_nparams; i < nparams; i++)
  {
    transform->variables[i] = NULL;
    transform->values[i] = (GLfloat *)malloc(sizeof(GLfloat));
    *transform->values[i] = 0.0;
  }
}

/* Get the type of the transform */
GlassTransformType _glassTransformGetType(GlassTransform *transform)
{
  return transform->type;
}

/* Set one of the parameters to a constant */
/* `param' is the parameter for the transform (starting from 0) */
void _glassTransformSetConstant(GlassTransform *transform, int param, GLfloat value)
{
  int nparams;

  nparams = _glassTransformGetnParams(transform->type);

  /* Invalid parameter */
  if(param >= nparams || param < 0)
  {
    fprintf(stderr, "Glass Error: Attempting to access parameter %d, which doesn't exist\n",
	    param);
    return;
  }

  /* Mark this parameter as not being controlled by a variable */
  transform->variables[param] = NULL;

  /* Make room for a constant */
  if(!transform->values[param])
    transform->values[param] = (GLfloat *)malloc(sizeof(GLfloat));
  *transform->values[param] = value;
}

/* Set one of the parameters to a variable */
/* `param' is the parameter for the transform (starting from 0) */
void _glassTransformSetVariable(GlassTransform *transform, int param, GlassVariable *variable)
{
  int nparams;

  nparams = _glassTransformGetnParams(transform->type);

  /* Invalid parameter */
  if(param >= nparams || param < 0)
  {
    fprintf(stderr, "Glass Error: Attempting to access parameter %d, which doesn't exist\n",
	    param);
    return;
  }

  /* If was controlled by a constant, remove that constant */
  if(transform->variables[param] == NULL)
  {
    free(transform->values[param]);
    transform->values[param] = NULL;
  }

  /* Point at the transform, and its value */
  transform->variables[param] = variable;
  transform->values[param] = (GLfloat *)_glassVariableGetValuev(variable);
}

/* Free a transform */
void _glassFreeTransform(GlassTransform *transform)
{
  int i, nparams;

  nparams = _glassTransformGetnParams(transform->type);

  for(i = 0; i < nparams; i++)
    if(!transform->variables[i])
      free(transform->values[i]);

  free(transform->variables);
  free(transform->values);

  free(transform);
}
