/* 
 * glass - the openGL Articulated Skeleton System
 *
 * Copyright (C) 2001-2003  Robert Ancell <bob27@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <malloc.h>
#include <math.h>

#include "global.h"
#include "glass_triangle.h"
#include "glass_surface.h"

/* Returns an empty triangle */
GlassTriangle *_glassTriangleNew(GlassSurface *surface)
{
  GlassTriangle *triangle;
  int i, j;

  triangle = (GlassTriangle *)malloc(sizeof(GlassTriangle));
  if(triangle == NULL)
  {
    fprintf(stderr, "Glass Error: Unable to allocate space for triangle\n");
    return NULL;
  }
  memset(triangle, 0, sizeof(GlassTriangle));

  /* Set the defaults */
  for(i = 0; i < 3; i++)
  {
    for(j = 0; j < 4; j++)
      triangle->col[i][j] = 1.0;

    triangle->nor[i][2] = 1.0;
  }

  if(surface->last_triangle)
  {
    surface->last_triangle->next = triangle;
    surface->last_triangle = triangle;
  }
  else
  {
    surface->triangles = triangle;
    surface->last_triangle = triangle;
  }

  return triangle;
}

/* Set the position for a triangles vertex */
void _glassTriangleSetPosition(GlassTriangle *triangle, int vertex, GLfloat *pos)
{
  GLfloat *vpos = triangle->pos[vertex];

  vpos[0] = pos[0];
  vpos[1] = pos[1];
  vpos[2] = pos[2];
}

/* Get the position of a triangles vertex */
GLfloat *_glassTriangleGetPosition(GlassTriangle *triangle, int vertex)
{
  return triangle->pos[vertex];
}

/* Set the colour for a triangles vertex */
void _glassTriangleSetColour(GlassTriangle *triangle, int vertex, GLfloat *col)
{
  GLfloat *vcol = triangle->col[vertex];
  int i;

  vcol[0] = col[0];
  vcol[1] = col[1];
  vcol[2] = col[2];
  if(triangle->is_alpha)
    vcol[3] = col[3];

  /* Clip the values */
  for(i = 0; i < 3; i++)
    if(vcol[i] < 0.0)
      vcol[i] = 0.0;
    else if(vcol[i] > 1.0)
      vcol[i] = 1.0;

  if(triangle->is_alpha)
  {
    if(vcol[3] < 0.0)
      vcol[3] = 0.0;
    else if(vcol[3] > 1.0)
      vcol[3] = 1.0;
  }
}

/* Get the colour of a triangles vertex */
GLfloat *_glassTriangleGetColour(GlassTriangle *triangle, int vertex)
{
  return triangle->col[vertex];
}

/* Set the normal for a triangles vertex */
void _glassTriangleSetNormal(GlassTriangle *triangle, int vertex, GLfloat *nor)
{
  GLfloat *vnor = triangle->nor[vertex], len;

  len = sqrt(nor[0]*nor[0] + nor[1]*nor[1] + nor[2]*nor[2]);

  vnor[0] = nor[0] / len;
  vnor[1] = nor[1] / len;
  vnor[2] = nor[2] / len;
}

/* Get the normal of a triangles vertex */
GLfloat *_glassTriangleGetNormal(GlassTriangle *triangle, int vertex)
{
  return triangle->nor[vertex];
}

/* Set the texture co-ordinates for a triangles vertex */
void _glassTriangleSetTexCoord(GlassTriangle *triangle, int vertex, GLfloat *tex)
{
  GLfloat *vtex = triangle->tex[vertex];

  vtex[0] = tex[0];
  vtex[1] = tex[1];
}

/* Get the texture co-ordinate of a triangles vertex */
GLfloat *_glassTriangleGetTexCoord(GlassTriangle *triangle, int vertex)
{
  return triangle->tex[vertex];
}

/* Set the material of the triangle */
void _glassTriangleSetMaterial(GlassTriangle *triangle, GlassMaterial *material)
{
  if(!material)
  {
    triangle->material = NULL;
    return;
  }

  if(triangle->is_alpha != material->is_alpha)
    fprintf(stderr,
	    "Glass Warning: Alpha material attached to non-alpha triangle, or visa versa\n");

  triangle->material = material;
}

/* Get the material of a triangle */
GlassMaterial *_glassTriangleGetMaterial(GlassTriangle *triangle)
{
  return triangle->material;
}

/* Free a triangle */
void _glassFreeTriangle(GlassTriangle *triangle)
{
  free(triangle);
}
